"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _awsSdk = require("aws-sdk");

var _bluebird = _interopRequireDefault(require("bluebird"));

var _retry = require("../utils/retry");

/* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */
_awsSdk.config.setPromisesDependency(_bluebird.default);

class Connector {
  constructor({
    debug,
    timeout = Number(process.env.BUS_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
    retryConfig = _retry.defaultRetryConfig
  }) {
    this.debug = msg => debug('%j', msg);

    this.bus = new _awsSdk.EventBridge({
      httpOptions: {
        timeout,
        connectTimeout: timeout
      },
      maxRetries: 10,
      // Default: 3
      retryDelayOptions: {
        base: 200
      },
      // Default: 100 ms
      logger: {
        log:
        /* istanbul ignore next */
        msg => debug('%s', msg.replace(/\n/g, '\r'))
      }
    });
    this.retryConfig = retryConfig;
  }

  putEvents(params) {
    return this._putEvents(params, []);
  }

  _putEvents(params, attempts) {
    (0, _retry.assertMaxRetries)(attempts, this.retryConfig.maxRetries);
    return (0, _retry.wait)((0, _retry.getDelay)(this.retryConfig.retryWait, attempts.length)).then(() => this.bus.putEvents(params).promise().tap(this.debug).tapCatch(this.debug).then(resp => {
      if (resp.FailedEntryCount > 0) {
        return this._putEvents(unprocessed(params, resp), [...attempts, resp]);
      } else {
        return accumlate(attempts, resp);
      }
    }));
  }

}

var _default = Connector;
exports.default = _default;

const unprocessed = (params, resp) => ({ ...params,
  Entries: params.Entries.filter((e, i) => resp.Entries[i].ErrorCode)
});

const accumlate = (attempts, resp) => attempts.reduceRight((a, c) => ({ ...a,
  Entries: [...c.Entries.filter(e => !e.ErrorCode), ...a.Entries.filter(e => !e.ErrorCode)],
  attempts: [...attempts, resp]
}), resp);