"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.scanDynamoDB = exports.toGetRequest = exports.toIndexQueryRequest = exports.toPkQueryRequest = exports.query = exports.queryDynamoDB = exports.batchGet = exports.batchGetDynamoDB = exports.put = exports.putDynamoDB = exports.update = exports.updateDynamoDB = exports.pkCondition = exports.timestampCondition = exports.updateExpression = void 0;

var _highland = _interopRequireDefault(require("highland"));

var _merge = _interopRequireDefault(require("lodash/merge"));

var _memoryCache = _interopRequireDefault(require("memory-cache"));

var _dynamodb = _interopRequireDefault(require("../connectors/dynamodb"));

var _faults = require("./faults");

var _print = require("./print");

const updateExpression = Item => {
  const keys = Object.keys(Item);
  const ExpressionAttributeNames = keys.filter(attrName => Item[attrName] !== undefined).map(attrName => ({
    [`#${attrName}`]: attrName
  })).reduce(_merge.default, {});
  const ExpressionAttributeValues = keys.filter(attrName => Item[attrName] !== undefined && Item[attrName] !== null).map(attrName => ({
    [`:${attrName}`]: Item[attrName]
  })).reduce(_merge.default, {});
  let UpdateExpression = `SET ${keys.filter(attrName => Item[attrName] !== undefined && Item[attrName] !== null).map(attrName => `#${attrName} = :${attrName}`).join(', ')}`;
  const UpdateExpressionRemove = keys.filter(attrName => Item[attrName] === null).map(attrName => `#${attrName}`).join(', ');

  if (UpdateExpressionRemove.length) {
    UpdateExpression = `${UpdateExpression} REMOVE ${UpdateExpressionRemove}`;
  }

  return {
    ExpressionAttributeNames,
    ExpressionAttributeValues,
    UpdateExpression,
    ReturnValues: 'ALL_NEW'
  };
};

exports.updateExpression = updateExpression;

const timestampCondition = (fieldName = 'timestamp') => ({
  ConditionExpression: `attribute_not_exists(#${fieldName}) OR #${fieldName} < :${fieldName}`
});

exports.timestampCondition = timestampCondition;

const pkCondition = (fieldName = 'pk') => ({
  ConditionExpression: `attribute_not_exists(${fieldName})`
});

exports.pkCondition = pkCondition;

const updateDynamoDB = ({
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.ENTITY_TABLE_NAME || process.env.EVENT_TABLE_NAME,
  updateRequestField = 'updateRequest',
  parallel = Number(process.env.UPDATE_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName,
    timeout
  });

  const invoke = uow => {
    if (!uow[updateRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = connector.update(uow[updateRequestField]).then(updateResponse => ({ ...uow,
      updateResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

exports.updateDynamoDB = updateDynamoDB;
const update = updateDynamoDB; // deprecated export

exports.update = update;

const putDynamoDB = ({
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME || process.env.ENTITY_TABLE_NAME,
  putRequestField = 'putRequest',
  parallel = Number(process.env.UPDATE_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName,
    timeout
  });

  const invoke = uow => {
    if (!uow[putRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = connector.put(uow[putRequestField]).then(putResponse => ({ ...uow,
      putResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

exports.putDynamoDB = putDynamoDB;
const put = putDynamoDB; // deprecated export

exports.put = put;

const batchGetDynamoDB = ({
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME || process.env.ENTITY_TABLE_NAME,
  batchGetRequestField = 'batchGetRequest',
  batchGetResponseField = 'batchGetResponse',
  parallel = Number(process.env.GET_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName,
    timeout
  });

  const invoke = uow => {
    if (!uow[batchGetRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = connector.batchGet(uow[batchGetRequestField]).then(batchGetResponse => ({ ...uow,
      [batchGetResponseField]: batchGetResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

exports.batchGetDynamoDB = batchGetDynamoDB;
const batchGet = batchGetDynamoDB; // deprecated export

exports.batchGet = batchGet;

const queryDynamoDB = (
/* istanbul ignore next */
{
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME || process.env.ENTITY_TABLE_NAME,
  queryRequestField = 'queryRequest',
  queryResponseField = 'queryResponse',
  parallel = Number(process.env.QUERY_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName,
    timeout
  });

  const invoke = uow => {
    if (!uow[queryRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const req = JSON.stringify(uow[queryRequestField]);

    const cached = _memoryCache.default.get(req);

    const p = (cached ? Promise.resolve(cached) : connector.query(uow[queryRequestField]).then(queryResponse => {
      _memoryCache.default.put(req, queryResponse);

      return queryResponse;
    })).then(queryResponse => ({ ...uow,
      [queryResponseField]: queryResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

exports.queryDynamoDB = queryDynamoDB;
const query = queryDynamoDB; // deprecated export

exports.query = query;

const toPkQueryRequest = (uow, rule) => ({
  KeyConditionExpression: '#pk = :pk',
  ExpressionAttributeNames: {
    '#pk': rule.pkFn || 'pk'
  },
  ExpressionAttributeValues: {
    ':pk': uow.event.partitionKey
  },
  ConsistentRead: true
});

exports.toPkQueryRequest = toPkQueryRequest;

const toIndexQueryRequest = (uow, rule) => ({
  IndexName: rule.indexNm,
  KeyConditionExpression: '#pk = :pk',
  ExpressionAttributeNames: {
    '#pk': rule.indexFn
  },
  ExpressionAttributeValues: {
    ':pk': uow.event.partitionKey
  },
  ConsistentRead: false
});

exports.toIndexQueryRequest = toIndexQueryRequest;

const toGetRequest = (uow, rule) => {
  const data = uow.event.raw.new ||
  /* istanbul ignore next */
  uow.event.raw.old;
  const Keys = rule.fks.reduce((a, fk) => {
    const value = data[fk];
    /* istanbul ignore else */

    if (value) {
      const [discriminator, pk] = value.split('|');
      return [...a, {
        pk,
        sk: discriminator
      }];
    } else {
      return a;
    }
  }, []);
  return {
    RequestItems: {
      [rule.tableName]: {
        Keys
      }
    }
  };
};

exports.toGetRequest = toGetRequest;

const scanDynamoDB = ({
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME || process.env.ENTITY_TABLE_NAME || process.env.TABLE_NAME,
  scanRequestField = 'scanRequest',
  scanResponseField = 'scanResponse',
  parallel = Number(process.env.SCAN_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName,
    timeout
  });

  const scan = uow => {
    let cursor;
    return (0, _highland.default)((push, next) => {
      const params = { ...uow[scanRequestField],
        ExclusiveStartKey: cursor
      };
      connector.scan(params).then(data => {
        const {
          LastEvaluatedKey,
          Items,
          ...rest
        } = data;

        if (LastEvaluatedKey) {
          cursor = LastEvaluatedKey;
        } else {
          cursor = undefined;
        }

        Items.forEach(Item => {
          push(null, { ...uow,
            [scanRequestField]: params,
            [scanResponseField]: { ...rest,
              Item
            }
          });
        });
      }).catch(
      /* istanbul ignore next */
      err => {
        err.uow = uow;
        push(err, null);
      }).finally(() => {
        if (cursor) {
          next();
        } else {
          push(null, _highland.default.nil);
        }
      });
    });
  };

  return s => s.map(scan).parallel(parallel);
};

exports.scanDynamoDB = scanDynamoDB;