"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sendToSqs = void 0;

var _highland = _interopRequireDefault(require("highland"));

var _sqs = _interopRequireDefault(require("../connectors/sqs"));

var _batch = require("./batch");

var _faults = require("./faults");

var _print = require("./print");

const sendToSqs = ({
  // eslint-disable-line import/prefer-default-export
  debug = (0, _print.debug)('sqs'),
  queueUrl = process.env.QUEUE_URL,
  messageField = 'message',
  batchSize = Number(process.env.SQS_BATCH_SIZE) || Number(process.env.BATCH_SIZE) || 10,
  parallel = Number(process.env.SQS_PARALLEL) || Number(process.env.PARALLEL) || 8
} = {}) => {
  const connector = new _sqs.default({
    debug,
    queueUrl
  });

  const toInputParams = batchUow => ({ ...batchUow,
    inputParams: {
      Entries: batchUow.batch.filter(uow => uow[messageField]).map(uow => uow[messageField])
    }
  });

  const sendMessageBatch = batchUow => {
    /* istanbul ignore next */
    if (!batchUow.inputParams.Entries.length) {
      return (0, _highland.default)(Promise.resolve(batchUow));
    }

    const p = connector.sendMessageBatch(batchUow.inputParams).then(sendMessageBatchResponse => ({ ...batchUow,
      sendMessageBatchResponse
    })).catch((0, _faults.rejectWithFault)(batchUow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.batch(batchSize).map(_batch.toBatchUow).map(toInputParams).map(sendMessageBatch).parallel(parallel).flatMap(_batch.unBatchUow); // for cleaner logging and testing
};

exports.sendToSqs = sendToSqs;